@@ -24,11 +24,12 @@
 #ifndef MODUTILS_OBJ_H
 #define MODUTILS_OBJ_H 1
 
-#ident "$Id: obj.h 1.2 Mon, 26 Feb 2001 11:59:14 +1100 kaos $"
+#ident "$Id: obj.h 1.3 Sun, 06 May 2001 14:23:41 +1000 kaos $"
 
 /* The relocatable object is manipulated using elfin types.  */
 
 #include <stdio.h>
+#include <sys/types.h>
 #include <elf.h>
 #include ELF_MACHINE_H
 
@@ -264,7 +265,7 @@ union obj_ptr_4 {
 	void *ptr;
 };
 union obj_ptr_8 {
-	uint64_t addr;	/* Should be Elf64_Xword but not all users have this yet */
+	u_int64_t addr;	/* Should be Elf64_Xword but not all users have this yet */
 	void *ptr;
 };
 
