@@ -33,7 +33,7 @@
  *   Keith Owens <kaos@ocs.com.au> November 2000.
  */
 
-#ident "$Id: modinfo.c 1.3 Sun, 25 Mar 2001 21:13:48 +1000 kaos $"
+#ident "$Id: modinfo.c 1.6 Wed, 28 Mar 2001 20:49:06 +1000 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -131,23 +131,70 @@ static const char *get_modinfo_value(struct obj_file *f, const char *key)
 	return NULL;
 }
 
+/* Most fields are fixed format which makes it relatively easy for programs to
+ * parse the output from modinfo.  But description and author fields can be
+ * multi line and can contain any characters.  To make it possible to parse the
+ * modinfo output, fields that might be multi line are printed enclosed in '"'
+ * for human readability and embedded newlines are printed as "\n  " so the
+ * second and subsequent lines are always indented.
+ */
+
 static int append_modinfo_tag(struct obj_file *f, char *tag, char *def,
-			      char **out_str, int n)
+			      char **out_str, int n, int multi_line)
 {
-	const char *p;
+	const char *p, *p1;
+	int extra = 0, l;
 
 	p = get_modinfo_value(f, tag);
 	if (!p && !def)
 		return -1;
 
-	if (!p)
+	if (!p) {
 		p = def;
+		multi_line = 0;
+	}
 
-	if (strlen(p) < n)
-		n = strlen(p);
+	if (multi_line) {
+		p1 = p;
+		extra = 2;
+		while ((p1 = strchr(p1, '\n'))) {
+			extra += 2;
+			++p1;
+		}
+	}
 
-	strncpy(*out_str, p, n);
-	*out_str += n;
+	if (strlen(p)+extra < n)
+		n = strlen(p)+extra;
+
+	if (!multi_line || n <= 2) {
+		strncpy(*out_str, p, n);
+		*out_str += n;
+	}
+	else {
+		*(*out_str)++ = '"';
+		n -= 2;
+		p1 = p;
+		while (*p1) {
+			p1 = strchr(p, '\n');
+			if (p1)
+				++p1;
+			else
+				p1 = p + strlen(p);
+			l = p1 - p;
+			if (l > n)
+				l = n;
+			strncpy(*out_str, p, l);
+			n -= l;
+			*out_str += l;
+			if (*(p1-1) == '\n' && n >= 2) {
+				strncpy(*out_str, "  ", 2);
+				*out_str += 2;
+				n -= 2;
+			}
+			p = p1;
+		}
+		*(*out_str)++ = '"';
+	}
 
 	return 0;
 }
@@ -189,21 +236,22 @@ static char *format_query_string(struct obj_file *f, const char *format)
 
 		case 'a':
 			(void) append_modinfo_tag(f, "author", "<none>",
-				&out_str, last_char - out_str);
+				&out_str, last_char - out_str, 1);
 			break;
 
 		case 'd':
 			(void) append_modinfo_tag(f, "description", "<none>",
-				&out_str, last_char - out_str);
+				&out_str, last_char - out_str, 1);
 			break;
 
 		case 'n':
 			(void) append_modinfo_tag(f, "filename", "<none>",
-				&out_str, last_char - out_str);
+				&out_str, last_char - out_str, 0);
 			break;
 
 		case '{':{
 				char tag[128], *end = strchr(in_str, '}');
+				int multi_line;
 
 				/* Make sure the %{...} is formatted correctly. */
 				if (!end) {
@@ -218,9 +266,11 @@ static char *format_query_string(struct obj_file *f, const char *format)
 				memset(tag, 0, sizeof(tag));
 				strncpy(tag, in_str, end - in_str);
 				tag[end - in_str] = '\0';
+				multi_line = strcmp(tag, "author") == 0 ||
+					     strcmp(tag, "description") == 0;
 
 				/* Append the tag's value if it exists. */
-				append_modinfo_tag(f, tag, "<none>", &out_str, last_char - out_str);
+				append_modinfo_tag(f, tag, "<none>", &out_str, last_char - out_str, multi_line);
 
 				/* Advance past the end of the replacement. */
 				in_str = end + 1;
@@ -260,7 +310,7 @@ static void show_parameter(struct obj_file *f, char *key, char *value,
 	if (max < min)
 		printf("warning: parameter %s has max < min!\n", key);
 
-	printf("%s ", key);
+	printf("parm:        %s ", key);
 
 	switch (*p) {
 	case 'c':
@@ -308,8 +358,23 @@ static void show_parameter(struct obj_file *f, char *key, char *value,
 	if (min > 1 || max > 1)
 		printf(" array (min = %d, max = %d)", min, max);
 
-	if (desc)
-		printf(", description \"%s\"", desc);
+	if (desc && *desc) {
+		const char *p = desc, *p1;
+		printf(", description \"");
+		p1 = p;
+		while (*p1) {
+			p1 = strchr(p, '\n');
+			if (p1)
+				++p1;
+			else
+				p1 = p + strlen(p);
+			printf("%.*s", p1 - p, p);
+			if (*(p1-1) == '\n')
+				printf("  ");
+			p = p1;
+		}
+		printf("\"");
+	}
 
 	printf("\n");
 }
@@ -383,14 +448,14 @@ int SHOW_MODULE_INFO(const char *filename, const char *fmtstr, int do_parameters
 		return -1;
 	gzf_close(fp);
 
-	if (do_parameters) {
-		show_module_parameters(f);
-	}
 	if (fmtstr) {
 		p = format_query_string(f, fmtstr);
 		if (p)
 			fputs(p, stdout);
 	}
+	if (do_parameters) {
+		show_module_parameters(f);
+	}
 	return 0;
 }
 
@@ -442,7 +507,7 @@ int main(int argc, char *argv[])
 		{0, 0, 0, 0}
 	};
 	int do_parameters = 1, opt;
-	char *fmtstr = "filename    - %n\ndescription - %d\nauthor      - %a\n";
+	char *fmtstr = "filename:    %n\ndescription: %d\nauthor:      %a\n";
 
 	while ((opt = getopt_long(argc, argv, "adnq:pf:Vh", long_opts, NULL)) != EOF)
 		switch (opt) {
