@@ -103,7 +103,7 @@ char *aliaslist[] =
 	"char-major-10-131 wdt",	/* /dev/temperature Machine internal temperature */
 					/* /dev/hwtrap Hardware fault trap */
 					/* /dev/exttrp External device trap */
-	"char-major-10-135 off",	/* rtc cannot be compiled as a module */
+	"char-major-10-135 rtc",	/* /dev/rtc Real time clock */
 	"char-major-10-139 openprom",	/* /dev/openprom Linux/Sparc interface */
 	"char-major-10-144 nvram",	/* from Tigran Aivazian <tigran@sco.COM> */
 	"char-major-10-157 applicom",	/* from David Woodhouse <dwmw2@infradead.org> */
